@@ -3,7 +3,7 @@
 #  Date:  18-May-2021 jdw
 #
 #  Updates:
-#
+#   18-Apr-2022  dwp Update getSupersededBy method to recursively return all superseded entries
 ##
 """Provide an inventory of removed repository content.
 """
@@ -51,13 +51,22 @@ class RemovedHoldingsProvider(object):
     def getSupersededBy(self, entryId):
         """Return the superseding entry ids"""
         try:
+            sL = []
             if isinstance(self.__invD[entryId.upper()]["superseded_by"], str):
-                return [self.__invD[entryId.upper()]["superseded_by"]]
+                sL = [self.__invD[entryId.upper()]["superseded_by"]]
             else:
-                return self.__invD[entryId.upper()]["superseded_by"]
+                sL = self.__invD[entryId.upper()]["superseded_by"]
+            if len(sL) > 0:
+                for recursiveEntry in sL:
+                    if isinstance(self.__invD[recursiveEntry.upper()]["superseded_by"], str):
+                        sL = sL + [self.__invD[recursiveEntry.upper()]["superseded_by"]]
+                    elif isinstance(self.__invD[recursiveEntry.upper()]["superseded_by"], list):
+                        sL = sL + self.__invD[recursiveEntry.upper()]["superseded_by"]
+                    else:
+                        break
         except Exception as e:
             logger.debug("Failing for %r with %s", entryId, str(e))
-        return []
+        return sL
 
     def getRemovedEntries(self):
         return list(self.__invD.keys())
@@ -108,6 +117,7 @@ class RemovedHoldingsProvider(object):
 
     def getAllContentTypes(self):
         """Return the removed content types for the input entry identifier"""
+        entryId = None
         try:
             cS = set()
             for entryId, tD in self.__invD.items():
@@ -128,7 +138,7 @@ class RemovedHoldingsProvider(object):
         #
         if useCache and self.__mU.exists(fp):
             invD = self.__mU.doImport(fp, fmt="json")
-            logger.debug("Reading cached inventory (%d)", len(invD))
+            logger.info("Reading cached inventory (%d) from file %s", len(invD), fp)
         else:
             logger.info("Fetch inventory from %s", urlTarget)
             ok = fU.get(urlTarget, fp)
@@ -155,7 +165,6 @@ class RemovedHoldingsProvider(object):
                         rmD[entryId] = {"status": "REMOVED", "status_code": "OBS", "id_code_replaced_by_latest": sL[-1]}
                     else:
                         rmD[entryId] = {"status": "REMOVED", "status_code": "OBS"}
-
                 else:
                     rmD[entryId] = {"status": "REMOVED", "status_code": "OBS"}
         return rmD
@@ -185,18 +194,13 @@ class RemovedHoldingsProvider(object):
             dict: dictionary of all removed data
             dict: dictionary of superseding entries
         """
+        # for transferred (which are the theoretical models, only pdb format is available):
         ct1MapD = {
-            "combined_nmr_data_NEF": "Combined NMR data (NEF)",
-            "combined_nmr_data_nmr-star": "Combined NMR data (NMR-STAR)",
-            "nmr_chemical_shifts": "NMR chemical shifts",
-            "nmr_restraints_v1": "NMR restraints",
-            "mmcif": "coordinates",
             "pdb": "coordinates",
-            "pdbml": "coordinates",
-            "structure_factors": "structure factors",
         }
+        # for removed:
         ct2MapD = {
-            "combined_nmr_data_NEF": "Combined NMR data (NEF)",
+            "combined_nmr_data_nef": "Combined NMR data (NEF)",
             "combined_nmr_data_nmr-star": "Combined NMR data (NMR-STAR)",
             "nmr_chemical_shifts": "NMR chemical shifts",
             "nmr_restraints_v1": "NMR restraints V1",
@@ -214,18 +218,18 @@ class RemovedHoldingsProvider(object):
             "status_code": "status_code",
             "release_date": "release_date",
             "remote_repository_name": "remote_repository_name",
-            # "remote_repository_title": "remote_repository_title",
-            # "content_type": "repository_content_types",
+            "remote_repository_title": "title",
+            "content_type": "repository_content_types",
             "deposit_date": "deposit_date",
             "remote_accession_code": "remote_accession_code",
-            "title": "title",
+            # "title": "title",
         }
         insMapD = {
             "deposition_authors": "audit_authors",
             "deposit_date": "deposit_date",
             "superseded_by": "id_codes_replaced_by",
             "release_date": "release_date",
-            "remove_date": "remove_date",
+            "obsolete_date": "remove_date",
             "status_code": "status_code",
             "title": "title",
         }
@@ -235,11 +239,11 @@ class RemovedHoldingsProvider(object):
             "details": "details",
             "superseded_by": "id_codes_replaced_by",
             "release_date": "release_date",
-            "remove_date": "remove_date",
-            # "content_type": "repository_content_types",
+            "obsolete_date": "remove_date",
+            "content_type": "repository_content_types",
             "title": "title",
         }
-        dateFields = ["deposit_date", "release_date", "obsolete_date", "remove_date"]
+        dateFields = ["deposit_date", "release_date", "remove_date"]
         trsfD = {}
         insilicoD = {}
         auditAuthorD = {}
@@ -343,7 +347,9 @@ class RemovedHoldingsProvider(object):
             #  ---- superseded ----
             if entryId in replacesD:
                 superD[entryId] = {"id_codes_superseded": replacesD[entryId]}
-
+        logger.info("# of transferred entries: %d", len(trsfD))
+        logger.info("# of insilico entries: %d", len(insilicoD))
+        logger.info("# of removed entries: %d", len(removedD))
         return trsfD, insilicoD, auditAuthorD, removedD, superD
 
     def __isContentInsilico(self, ctD):
