@@ -25,6 +25,8 @@
 #    8-Oct-2021  jdw pass configuration URLs to CurrentHoldingsProvider and RemoveHoldingsProvider
 #                    ValidationReportProvider() migrated to ValidationReportAdapter()
 #    8-Oct-2021  jdw add warning messages for empty read/merge container results in method __mergeContainers()
+#    5-Apr-2022  dwp Add support for loading id code lists for bird_chem_comp_core (mainly used for Azure testing)
+#   13-Apr-2022  dwp Update methods for obtaining list of computed-model files
 ##
 """
 Utilities for scanning and accessing data in PDBx/mmCIF data in common repository file systems or via remote repository services.
@@ -35,13 +37,13 @@ __author__ = "John Westbrook"
 __email__ = "jwest@rcsb.rutgers.edu"
 __license__ = "Apache 2.0"
 
-import glob
 import logging
 import os
 import time
 
 from rcsb.utils.io.HashableDict import HashableDict
 from rcsb.utils.io.MarshalUtil import MarshalUtil
+from rcsb.utils.io.FileUtil import FileUtil
 from rcsb.utils.multiproc.MultiProcUtil import MultiProcUtil
 from rcsb.utils.repository.CurrentHoldingsProvider import CurrentHoldingsProvider
 from rcsb.utils.repository.RemovedHoldingsProvider import RemovedHoldingsProvider
@@ -99,6 +101,7 @@ class RepositoryProvider(object):
         self.__cachePath = os.path.join(self.__topCachePath, "repo_util")
         #
         self.__mU = MarshalUtil(workPath=self.__cachePath)
+        self.__fU = FileUtil(workPath=self.__cachePath)
         self.__chP = None
         self.__rhP = None
         logger.info("Discovery mode is %r", self.__discoveryMode)
@@ -317,7 +320,7 @@ class RepositoryProvider(object):
             elif contentType in ["pdb_distro", "da_internal", "status_history"]:
                 outputLocatorList = inputPathList if inputPathList else []
             elif contentType in ["pdbx_comp_model_core"]:
-                outputLocatorList = inputPathList if inputPathList else self.__getModelPathList()
+                outputLocatorList = inputPathList if inputPathList else self.__getCompModelPathList()
             else:
                 logger.warning("Unsupported contentType %s", contentType)
         except Exception as e:
@@ -331,6 +334,7 @@ class RepositoryProvider(object):
     def __getLocatorListRemote(self, contentType, inputIdCodeList=None, mergeContentTypes=None):
         outputLocatorList = []
         idCodeList = inputIdCodeList if inputIdCodeList else []
+        logger.info("Getting remote locator list for contentType %s with idCodeList: %r", contentType, idCodeList)
         try:
             if contentType in ["bird", "bird_core"]:
                 outputLocatorList = self.__getBirdUriList(idCodeList=idCodeList)
@@ -340,8 +344,10 @@ class RepositoryProvider(object):
                 outputLocatorList = self.__getChemCompUriList(idCodeList=idCodeList)
             elif contentType in ["bird_chem_comp"]:
                 outputLocatorList = self.__getBirdChemCompUriList(idCodeList=idCodeList)
-            elif contentType in ["chem_comp_core", "bird_consolidated", "bird_chem_comp_core"]:
+            elif contentType in ["chem_comp_core", "bird_consolidated", "bird_chem_comp_core"] and not self.__fileLimit:
                 outputLocatorList = self.mergeBirdAndChemCompRefData()
+            elif contentType in ["chem_comp_core", "bird_consolidated", "bird_chem_comp_core"] and self.__fileLimit:
+                outputLocatorList = self.mergeBirdAndChemCompRefDataWithInput(idCodeList=idCodeList)
             #
             elif contentType in ["pdbx", "pdbx_core"]:
                 if mergeContentTypes and "vrpt" in mergeContentTypes:
@@ -357,15 +363,17 @@ class RepositoryProvider(object):
             elif contentType in ["ihm_dev", "ihm_dev_core", "ihm_dev_full"]:
                 outputLocatorList = self.__getIhmDevPathList()
             elif contentType in ["pdbx_comp_model_core"]:
-                outputLocatorList = self.__getModelPathList()
+                outputLocatorList = self.__getCompModelPathList()
             else:
                 logger.warning("Unsupported contentType %s", contentType)
 
         except Exception as e:
             logger.exception("Failing with %s", str(e))
 
+        logger.debug("outputLocatorList before applying fileLimit (%r): %r", self.__fileLimit, outputLocatorList)
         if self.__fileLimit:
             outputLocatorList = outputLocatorList[: self.__fileLimit]
+            logger.debug("outputLocatorList after applying fileLimit (%r): %r", self.__fileLimit, outputLocatorList)
 
         return sorted(outputLocatorList) if outputLocatorList and isinstance(outputLocatorList[0], str) else outputLocatorList
 
@@ -489,6 +497,8 @@ class RepositoryProvider(object):
                 pth = self.__cfgOb.getPath("PDBX_REPO_PATH", sectionName=self.__configName)
             elif contentType in ["pdbx_obsolete"]:
                 pth = self.__cfgOb.getPath("PDBX_OBSOLETE_REPO_PATH", sectionName=self.__configName)
+            elif contentType in ["pdbx_comp_model_core"]:
+                pth = self.__cfgOb.getPath("PDBX_COMP_MODEL_REPO_PATH", sectionName=self.__configName)
             elif contentType in ["bird_consolidated", "bird_chem_comp_core"]:
                 pth = self.__cachePath
             elif contentType in ["ihm_dev", "ihm_dev_core", "ihm_dev_full"]:
@@ -520,6 +530,7 @@ class RepositoryProvider(object):
                 tIdL = [idCode.upper() for idCode in idCodeList if idCode.upper() in tIdD]
                 # idCodeList = [t.upper() for t in idCodeList]
                 # tIdL = list(set(tIdL).intersection(idCodeList))
+                logger.info("idCodeList selected: %r", tIdL)
             #
             for tId in tIdL:
                 kwD = HashableDict({})
@@ -544,7 +555,7 @@ class RepositoryProvider(object):
             if idCodeList:
                 tIdD = dict.fromkeys(tIdL, True)
                 tIdL = [idCode.upper() for idCode in idCodeList if idCode.upper() in tIdD]
-
+                logger.info("idCodeList selected: %r", tIdL)
             #
             for tId in tIdL:
                 kwD = HashableDict({})
@@ -564,7 +575,7 @@ class RepositoryProvider(object):
             if idCodeList:
                 tIdD = dict.fromkeys(tIdL, True)
                 tIdL = [idCode.upper() for idCode in idCodeList if idCode.upper() in tIdD]
-
+                logger.info("idCodeList selected: %r", tIdL)
             #
             kwD = HashableDict({})
             for tId in tIdL:
@@ -583,7 +594,7 @@ class RepositoryProvider(object):
             if idCodeList:
                 tIdD = dict.fromkeys(tIdL, True)
                 tIdL = [idCode.upper() for idCode in idCodeList if idCode.upper() in tIdD]
-
+                logger.info("idCodeList selected: %r", tIdL)
             #
             kwD = HashableDict({})
             for tId in tIdL:
@@ -602,7 +613,7 @@ class RepositoryProvider(object):
             if idCodeList:
                 tIdD = dict.fromkeys(tIdL, True)
                 tIdL = [idCode.upper() for idCode in idCodeList if idCode.upper() in tIdD]
-
+                logger.info("idCodeList selected: %r", tIdL)
             #
             kwD = HashableDict({})
             for tId in tIdL:
@@ -621,6 +632,7 @@ class RepositoryProvider(object):
             if idCodeList:
                 tIdD = dict.fromkeys(tIdL, True)
                 tIdL = [idCode.upper() for idCode in idCodeList if idCode.upper() in tIdD]
+                logger.info("idCodeList selected: %r", tIdL)
             #
             kwD = HashableDict({})
             for tId in tIdL:
@@ -748,6 +760,24 @@ class RepositoryProvider(object):
                         pathList.append(os.path.join(root, name))
         return dataList, pathList, []
 
+    def _compModelPathWorker(self, dataList, procName, optionsD, workingDir):
+        """Return the list of computed-model entry file paths in the current computed-model storage area.
+
+        Args:
+            dataList (list): List of sub-directory paths to model files in the computed-model storage area, with paths starting at
+                             the model source prefix (e.g., ["AF/XJ/E6/AF_AFA0A385XJE6F1.cif.gz", "MA/PC/05/MA_MABAKCEPC0534.cif.gz."])
+            procName (str): worker process name
+            optionsD (dict): dictionary of additional options that worker can access
+            workingDir (str): path to working directory
+        """
+        _ = procName
+        _ = workingDir
+        topRepoPath = optionsD["topRepoPath"]
+        pathList = []
+        for modelPath in dataList:
+            pathList.append(os.path.join(topRepoPath, modelPath))
+        return dataList, pathList, []
+
     def __getEntryPathList(self):
         return self.__fetchEntryPathList(self.__getRepoLocalPath("pdbx"), numProc=self.__numProc)
 
@@ -904,7 +934,7 @@ class RepositoryProvider(object):
 
         return prdD
 
-    def __buildBirdCcIndex(self):
+    def __buildBirdCcIndex(self, idCodeList=None):
         """Using information from the PRD pdbx_reference_molecule category to
         index the BIRDs corresponding small molecule correspondences
 
@@ -913,16 +943,20 @@ class RepositoryProvider(object):
         ccPathD = {}
         prdStatusD = {}
         try:
-            ccPathL = self.getLocatorPaths(self.__getLocatorList("chem_comp"))
+            ccPathL = self.getLocatorPaths(self.__getLocatorList("chem_comp", inputIdCodeList=idCodeList))
+            logger.debug("ccPathL: %r", ccPathL)
             ccPathD = {}
             for ccPath in ccPathL:
                 _, fn = os.path.split(ccPath)
                 ccId, _ = os.path.splitext(fn)
                 ccPathD[ccId] = ccPath
             logger.info("Chemical component path list (%d)", len(ccPathD))
+            # logger.info("Chemical component path list: %r", ccPathD)
             #
-            pthL = self.getLocatorPaths(self.__getLocatorList("bird"))
+            pthL = self.getLocatorPaths(self.__getLocatorList("bird", inputIdCodeList=idCodeList))
             logger.info("BIRD path list (%d)", len(pthL))
+            # logger.info("BIRD path list: %r", pthL)
+            #
             for pth in pthL:
                 containerL = self.__mU.doImport(pth, fmt="mmcif")
                 for container in containerL:
@@ -955,15 +989,49 @@ class RepositoryProvider(object):
     # -
     def mergeBirdAndChemCompRefData(self):
         # JDW note that this merging procedure expects access to all reference data -
-        # self.__fileLimit = None
+        # Use this method when:  self.__fileLimit = None
         prdSmallMolCcD, ccPathD, prdStatusD = self.__buildBirdCcIndex()
         logger.info("PRD to CCD index length %d CCD map path length %d", len(prdSmallMolCcD), len(ccPathD))
         outputPathList = self.__mergeBirdRefData(prdSmallMolCcD, prdStatusD)
-        ccOutputPathList = [pth for pth in self.__getChemCompPathList() if pth not in ccPathD]
+        #
+        ccOutputPathList = []
+        if self.__discoveryMode == 'remote':
+            for pth in self.__getChemCompUriList(idCodeList=None):
+                ccp = pth[0]['locator']
+                if ccp not in ccPathD:
+                    ccOutputPathList.append(ccp)
+        else:
+            ccOutputPathList = [pth for pth in self.__getChemCompPathList() if pth not in ccPathD]
+        #
         outputPathList.extend(ccOutputPathList)
+        logger.info("Total cc paths: %d", len(ccOutputPathList))
+        logger.info("Total bird_chem_comp paths: %d", len(outputPathList))
+        #
         return outputPathList
 
-    def __mergeBirdRefData(self, prdSmallMolCcD, prdStatusD):
+    def mergeBirdAndChemCompRefDataWithInput(self, idCodeList=None):
+        # Use this method when:  self.__fileLimit != None
+        idCodeList = idCodeList if idCodeList else []
+        prdSmallMolCcD, ccPathD, prdStatusD = self.__buildBirdCcIndex(idCodeList=idCodeList)
+        logger.info("PRD to CCD index length %d CCD map path length %d", len(prdSmallMolCcD), len(ccPathD))
+        outputPathList = self.__mergeBirdRefData(prdSmallMolCcD, prdStatusD, idCodeList=idCodeList)
+        #
+        ccOutputPathList = []
+        if self.__discoveryMode == 'remote':
+            for pth in self.__getChemCompUriList(idCodeList=idCodeList):
+                ccp = pth[0]['locator']
+                if ccp not in ccPathD:
+                    ccOutputPathList.append(ccp)
+        else:
+            ccOutputPathList = [pth for pth in self.__getChemCompPathList() if pth not in ccPathD]
+        #
+        outputPathList.extend(ccOutputPathList)
+        logger.info("Total cc paths: %d", len(ccOutputPathList))
+        logger.info("Total bird_chem_comp paths: %d", len(outputPathList))
+        #
+        return outputPathList
+
+    def __mergeBirdRefData(self, prdSmallMolCcD, prdStatusD, idCodeList=None):
         """Consolidate all of the bird reference data in a single container.
 
         If the BIRD is a 'small molecule' type then also merge with the associated CC definition.
@@ -976,7 +1044,7 @@ class RepositoryProvider(object):
         outPathList = []
         iSkipUnreleased = 0
         try:
-            birdPathList = self.getLocatorPaths(self.__getLocatorList("bird"))
+            birdPathList = self.getLocatorPaths(self.__getLocatorList("bird", inputIdCodeList=idCodeList))
             birdPathD = {}
             for birdPath in birdPathList:
                 _, fn = os.path.split(birdPath)
@@ -985,7 +1053,7 @@ class RepositoryProvider(object):
             #
             logger.info("BIRD path length %d", len(birdPathD))
             logger.debug("BIRD keys %r", list(birdPathD.keys()))
-            birdCcPathList = self.getLocatorPaths(self.__getLocatorList("bird_chem_comp"))
+            birdCcPathList = self.getLocatorPaths(self.__getLocatorList("bird_chem_comp", inputIdCodeList=idCodeList))
             birdCcPathD = {}
             for birdCcPath in birdCcPathList:
                 _, fn = os.path.split(birdCcPath)
@@ -1106,34 +1174,92 @@ class RepositoryProvider(object):
         return outPathList
         #
 
-    def __getModelPathList(self):
-        """Return the list of computational models the current cached model repository.
+    def __getCompModelPathList(self):
+        return self.__fetchModelPathList(self.__getRepoLocalPath("pdbx_comp_model_core"), numProc=self.__numProc)
 
-        File name template is:  <modelDirPath>/<source>/*.cif.gz
-                                         -  or -
-                                <modelDirPath>/<hash>/<hash>/*.cif.gz
+    def __fetchModelPathList(self, topRepoPath, numProc=8):
+        """Get the path list for computational models in the input cached model repository
 
+        TO-DO: Add check of cache file to see if it changed between the last time data was uploaded, and if so, then upload new models
+
+        File name template is:  <topRepoPath>/<2-char source>/<hash>/<hash>/*.cif.gz
+        """
+        ts = time.strftime("%Y %m %d %H:%M:%S", time.localtime())
+        logger.info("Starting at %s", ts)
+        logger.info("Computed-models topRepoPath: %s", topRepoPath)
+        startTime = time.time()
+        #
+        pathList = []
+        try:
+            compModelCacheFile, cacheFmt, compressed = self.__getCompModelCachPath()
+            if not compModelCacheFile:
+                logger.info("Failed to determine path of computed-models cache file. Returning empty pathList.")
+                return pathList
+            if cacheFmt == "pickle" and compressed:
+                compModelCacheFile = self.__fU.uncompress(compModelCacheFile)
+            compModelCacheD = self.__mU.doImport(compModelCacheFile, fmt=cacheFmt)
+            dataList = []
+            for _, modelD in compModelCacheD.items():
+                dataList.append(modelD["modelPath"])
+            logger.info("Computed-models loaded dataList length: %d", len(dataList))
+            #
+            optD = {}
+            optD["topRepoPath"] = topRepoPath
+            mpu = MultiProcUtil(verbose=self.__verbose)
+            mpu.setOptions(optionsD=optD)
+            mpu.set(workerObj=self, workerMethod="_compModelPathWorker")
+            _, _, retLists, _ = mpu.runMulti(dataList=dataList, numProc=numProc, numResults=1)
+            pathList = retLists[0]
+            endTime0 = time.time()
+            logger.debug("Path list length %d  in %.4f seconds", len(pathList), endTime0 - startTime)
+        except Exception as e:
+            logger.exception("Failing with %s", str(e))
+        return self.__applyLimit(pathList)
+
+    def __getCompModelCachPath(self):
+        """Convenience method to return path for computed-model cache file (json or pickle),
+        which contains the list of all computed-models in storage area.
+        """
+        pth = None
+        fmt = None
+        compressed = False
+        try:
+            pth = self.__cfgOb.getPath("PDBX_COMP_MODEL_CACHE_LIST_PATH", sectionName=self.__configName)
+            if pth.endswith(".pic") or pth.endswith(".pic.gz"):
+                fmt = "pickle"
+            elif pth.endswith(".json") or pth.endswith(".json.gz"):
+                fmt = "json"
+            else:
+                logger.warning("Unsupported format/extension for computed-model cache file %s", pth)
+            if pth.endswith(".gz"):
+                compressed = True
+        except Exception as e:
+            logger.exception("Failing with %s", str(e))
+        return pth, fmt, compressed
+
+    def getCompModelIdMap(self):
+        return self.__fetchCompModelIdMap()
+
+    def __fetchCompModelIdMap(self):
+        """Get the ID mapping between the source model IDs and internal model identifiers for computational models.
         """
         #
-        for modelType in ["computed-models"]:
-            modelDirPath = os.path.join(self.__topCachePath, modelType)
-            pathList = []
-            logger.info("Searching for models in path %r", modelDirPath)
-            try:
-                pattern = os.path.join(modelDirPath, "*", "*.cif.gz")
-                logger.info("Using pattern %r", pattern)
-                for pth in glob.iglob(pattern, recursive=True):
-                    pathList.append(pth)
-                if not pathList:
-                    pattern = os.path.join(modelDirPath, "*", "*", "*", "*.cif.gz")
-                    logger.info("Using pattern %r", pattern)
-                    for pth in glob.iglob(pattern, recursive=True):
-                        pathList.append(pth)
-
-            except Exception as e:
-                logger.exception("Failing search in %r with %s", modelDirPath, str(e))
+        compModelIdMapD = {}
+        try:
+            compModelCacheFile, cacheFmt, compressed = self.__getCompModelCachPath()
+            if not compModelCacheFile:
+                logger.info("Failed to determine path of computed-models cache file. Returning empty compModelIdMapD.")
+                return compModelIdMapD
+            if cacheFmt == "pickle" and compressed:
+                compModelCacheFile = self.__fU.uncompress(compModelCacheFile)
+            compModelCacheD = self.__mU.doImport(compModelCacheFile, fmt=cacheFmt)
+            for internalModelId, modelD in compModelCacheD.items():
+                compModelIdMapD.update({modelD["sourceId"]: internalModelId})
+            logger.info("Computed-models mapped ID length: %d", len(compModelIdMapD))
             #
-        return self.__applyLimit(pathList)
+        except Exception as e:
+            logger.exception("Failing with %s", str(e))
+        return compModelIdMapD
 
     def __getIhmDevPathList(self):
         return self.__fetchIhmDevPathList(self.__getRepoLocalPath("ihm_dev"))
